@@ -1,5 +1,4 @@
 use std::collections::{HashMap, HashSet};
-use std::env;
 #[cfg(unix)]
 use std::os::unix::fs::MetadataExt;
 use std::path::Path;
@@ -7,23 +6,14 @@ use std::path::Path;
 use std::time::UNIX_EPOCH;
 
 use anyhow::{Context, Result};
-use chrono;
-use fs2;
 use log::{debug, info, warn};
 use tokio::fs;
 use tokio::io::AsyncReadExt;
+use tokio::time::{interval, Duration};
 
 use crate::constants::*;
 use crate::download_core::check_metadata_size;
 
-/// Get the cleanup threshold hours from environment variable
-fn get_b10fs_cleanup_threshold_hours() -> u64 {
-    env::var(TRUSS_TRANSFER_B10FS_CLEANUP_HOURS_ENV_VAR)
-        .ok()
-        .and_then(|s| s.parse().ok())
-        .unwrap_or(TRUSS_TRANSFER_B10FS_DEFAULT_CLEANUP_HOURS)
-}
-
 /// Helper function to get the file's last access time as a Unix timestamp.
 /// On Unix, it uses `metadata.atime()`. On Windows, it uses `metadata.accessed()`.
 fn get_atime(metadata: &std::fs::Metadata) -> std::io::Result<i64> {
@@ -43,31 +33,40 @@ fn get_atime(metadata: &std::fs::Metadata) -> std::io::Result<i64> {
 /// Returns a tuple: (available_volume_bytes, manifest_files_in_cache_bytes).
 /// Files are cleaned up if they:
 /// - Have not been accessed within the threshold AND are not in `current_hashes`
+/// - Are not in `current_hashes` and cache size exceeds TRUSS_TRANSFER_B10FS_MAX_STALE_CACHE_SIZE_GB
 pub async fn cleanup_b10cache_and_get_space_stats(
     current_hashes: &HashSet<String>,
     manifest_hash_to_size_map: &HashMap<String, u64>,
 ) -> Result<(u64, u64)> {
     // Returns (available_volume_bytes, manifest_files_in_cache_bytes)
-    let cleanup_threshold_hours = get_b10fs_cleanup_threshold_hours();
-    let cache_dir_path = Path::new(CACHE_DIR);
+    let cleanup_threshold_hours = *TRUSS_TRANSFER_B10FS_CLEANUP_HOURS;
+    let cache_dir_path = Path::new(&*CACHE_DIR);
     let now = chrono::Utc::now().timestamp();
     let threshold_seconds = cleanup_threshold_hours * 3600;
+    let max_stale_cache_size_gb = *crate::constants::TRUSS_TRANSFER_B10FS_MAX_STALE_CACHE_SIZE_GB;
 
     let mut dir = fs::read_dir(cache_dir_path)
         .await
-        .with_context(|| format!("Failed to read cache directory: {:?}", cache_dir_path))?;
+        .with_context(|| format!("Failed to read cache directory: {cache_dir_path:?}"))?;
 
     let mut total_bytes_managed_in_cache = 0u64; // All files kept in cache (manifest or not old enough)
     let mut total_files_managed_in_cache = 0usize;
     let mut manifest_files_in_cache_bytes = 0u64; // Size of files from current manifest, correctly cached
 
+    // For size-based cleanup: list of (access_time, file_path, size) for non-current files
+    let mut stale_files: Vec<(i64, std::path::PathBuf, u64)> = Vec::new();
+
     info!(
         "Analyzing b10cache at {} with a cleanup threshold of {} hours ({} days)",
-        CACHE_DIR,
+        &*CACHE_DIR,
         cleanup_threshold_hours,
         cleanup_threshold_hours as f64 / 24.0
     );
 
+    if let Some(max_size_gb) = max_stale_cache_size_gb {
+        info!("Max stale cache size limit: {} GB", max_size_gb);
+    }
+
     while let Some(entry) = dir.next_entry().await? {
         let path = entry.path();
         // Only process files
@@ -91,34 +90,82 @@ pub async fn cleanup_b10cache_and_get_space_stats(
                         // This file, though in manifest, is corrupted in cache.
                         // It won't be deleted by age here, but download logic might replace it if it tries to use it.
                     }
-                } else if now - atime > threshold_seconds as i64
-                    && !current_hashes.contains(file_name_hash)
-                {
-                    // File is NOT in current manifest AND is older than threshold
-                    info!(
-                        "Deleting old cached file {} ({} bytes): not in current manifest and not accessed for over {} hours",
-                        file_name_hash, actual_file_size, cleanup_threshold_hours
-                    );
-                    if let Err(e) = fs::remove_file(&path).await {
-                        warn!("Failed to delete cached file {:?}: {}", path, e);
-                    }
-                } else {
-                    // File is NOT in current manifest but is NOT old enough to delete OR
-                    // it IS in current_hashes but somehow not in manifest_hash_to_size_map (should not happen if maps are consistent)
-                    // or it was in manifest_hash_to_size_map but had wrong size (already handled above for manifest_files_in_cache_bytes)
-                    if !manifest_hash_to_size_map.contains_key(file_name_hash) {
-                        // only log if truly not in manifest
+                } else if !current_hashes.contains(file_name_hash) {
+                    // File is NOT in current_hashes - candidate for deletion
+                    if now - atime > threshold_seconds as i64 {
+                        // File is older than threshold - delete immediately
+                        info!(
+                            "Deleting old cached file {} ({} bytes): not in current manifest and not accessed for over {} hours",
+                            file_name_hash, actual_file_size, cleanup_threshold_hours
+                        );
+                        if let Err(e) = fs::remove_file(&path).await {
+                            warn!("Failed to delete cached file {:?}: {}", path, e);
+                        }
+                    } else {
+                        // File is not old enough for time-based cleanup, but might be cleaned up for size limit
+                        stale_files.push((atime, path.clone(), actual_file_size));
+                        total_bytes_managed_in_cache += actual_file_size;
+                        total_files_managed_in_cache += 1;
                         debug!(
                             "Keeping non-manifest file {} ({} bytes): last accessed {} minutes ago.",
                             file_name_hash,
                             actual_file_size,
                             (now - atime) / 60
                         );
-                        total_bytes_managed_in_cache += actual_file_size;
-                        total_files_managed_in_cache += 1;
+                    }
+                } else {
+                    // File is in current_hashes but not in manifest_hash_to_size_map
+                    // (should not happen if maps are consistent)
+                    total_bytes_managed_in_cache += actual_file_size;
+                    total_files_managed_in_cache += 1;
+                }
+            }
+        }
+    }
+
+    // Size-based cleanup if max_stale_cache_size_gb is set
+    if let Some(max_size_gb) = max_stale_cache_size_gb {
+        let max_size_bytes = max_size_gb * 1024 * 1024 * 1024; // Convert GB to bytes
+        let mut current_stale_size: u64 = stale_files.iter().map(|(_, _, size)| size).sum();
+
+        if current_stale_size > max_size_bytes {
+            info!(
+                "Stale cache size ({:.2} GB) exceeds limit ({} GB). Starting size-based cleanup.",
+                current_stale_size as f64 / (1024.0 * 1024.0 * 1024.0),
+                max_size_gb
+            );
+
+            // Sort by access time (oldest first)
+            stale_files.sort_by_key(|(atime, _, _)| *atime);
+
+            // Delete files starting from oldest access time until under the limit
+            for (atime, file_path, file_size) in stale_files.iter() {
+                if current_stale_size <= max_size_bytes {
+                    break;
+                }
+
+                if let Some(file_name) = file_path.file_name().and_then(|n| n.to_str()) {
+                    info!(
+                        "Deleting stale cached file {} ({} bytes) for size limit: last accessed {} hours ago",
+                        file_name,
+                        file_size,
+                        (now - atime) / 3600
+                    );
+
+                    if let Err(e) = fs::remove_file(file_path).await {
+                        warn!("Failed to delete cached file {:?}: {}", file_path, e);
+                    } else {
+                        current_stale_size -= file_size;
+                        total_bytes_managed_in_cache -= file_size;
+                        total_files_managed_in_cache -= 1;
                     }
                 }
             }
+
+            info!(
+                "Size-based cleanup complete. Stale cache size now: {:.2} GB",
+                current_stale_size as f64 / (1024.0 * 1024.0 * 1024.0)
+            );
         }
     }
 
@@ -131,12 +178,12 @@ pub async fn cleanup_b10cache_and_get_space_stats(
 
     // Get available disk space for CACHE_DIR's volume
     let stats = fs2::statvfs(cache_dir_path)
-        .with_context(|| format!("Failed to get volume stats for {:?}", cache_dir_path))?;
+        .with_context(|| format!("Failed to get volume stats for {cache_dir_path:?}"))?;
     let available_bytes = stats.available_space(); // f_bavail * f_frsize (available to non-root)
 
     info!(
         "Total available space on volume for {}: {:.2} GB ({} bytes)",
-        CACHE_DIR,
+        &*CACHE_DIR,
         available_bytes as f64 / (1024.0 * 1024.0 * 1024.0),
         available_bytes
     );
@@ -195,15 +242,70 @@ pub async fn handle_write_b10cache(download_path: &Path, cache_path: &Path) -> R
     }
 
     if should_copy {
-        // Copy the local file to the incomplete cache file.
+        // Copy the local file to the incomplete cache file with progress monitoring
         info!(
-            "Copying local file {:?} to temporary incomplete cache file {:?}",
+            "Copying local file {:?} to cache file {:?}",
             download_path, incomplete_cache_path
         );
-        match fs::copy(download_path, &incomplete_cache_path).await {
-            Ok(_) => info!("Successfully copied to incomplete cache file."),
+
+        // Setup progress monitoring with atomics
+        let monitor_path = incomplete_cache_path.to_path_buf();
+        let monitor_handle = tokio::spawn({
+            async move {
+                let mut ticker = interval(Duration::from_secs(30));
+                let mut last_size = 0;
+                loop {
+                    ticker.tick().await;
+                    let current_size = match fs::metadata(&monitor_path).await {
+                        Ok(metadata) => metadata.len(),
+                        Err(e) => {
+                            warn!(
+                                "Issue monitoring the copy: read metadata for {:?} failed: {}",
+                                monitor_path, e
+                            );
+                            continue;
+                        }
+                    };
+
+                    if current_size == last_size {
+                        warn!(
+                            "No progress for {:?}: {} bytes written so far.",
+                            monitor_path, current_size
+                        );
+                    } else {
+                        let progress_mb = current_size as f64 / (1024.0 * 1024.0);
+                        info!(
+                            "Copy progress for {:?}: {:.2} MB written so far.",
+                            monitor_path, progress_mb
+                        );
+                    }
+                    last_size = current_size;
+                }
+            }
+        });
+        let copy_result = match fs::rename(download_path, &incomplete_cache_path).await {
+            Ok(_) => {
+                info!("Successfully moved file to cache.");
+                Ok(0) // Using 0 as a placeholder for success, similar to how copy returns bytes.
+            }
+            Err(_) => {
+                //  Failed to move file, falling back to copy (likely cross-device)
+                let copy_result = fs::copy(download_path, &incomplete_cache_path).await;
+                info!("Successfully copied local file to incomplete cache.");
+                copy_result
+            }
+        };
+
+        // Stop monitoring regardless of copy result
+        monitor_handle.abort();
+
+        match copy_result {
+            Ok(_) => debug!("Successfully copied to incomplete cache file."),
             Err(e) => {
-                warn!("Failed to copy local file to incomplete cache file: {}. Maybe b10cache has no storage.", e);
+                warn!(
+                    "Failed to copy local file to incomplete cache file: {}. Maybe b10cache has no storage or permission issues.",
+                    e
+                );
                 return Ok(());
             }
         }
@@ -212,7 +314,10 @@ pub async fn handle_write_b10cache(download_path: &Path, cache_path: &Path) -> R
     let incomplete_metadata = match fs::metadata(&incomplete_cache_path).await {
         Ok(metadata) => metadata,
         Err(e) => {
-            warn!("Failed to get metadata for incomplete cache file: {}. Maybe b10cache has no storage or concurrency issue.", e);
+            warn!(
+                "Failed to get metadata for incomplete cache file: {}. Maybe b10cache has no storage or concurrency issue.",
+                e
+            );
             return Ok(());
         }
     };
@@ -231,24 +336,38 @@ pub async fn handle_write_b10cache(download_path: &Path, cache_path: &Path) -> R
     }
 
     // Atomically rename the incomplete file to the final cache file.
-    info!(
+    debug!(
         "Atomic rename: renaming incomplete cache file {:?} to final cache file {:?}",
         incomplete_cache_path, cache_path
     );
-    fs::rename(&incomplete_cache_path, cache_path)
-        .await
-        .with_context(|| {
-            format!(
-                "Failed to atomically rename incomplete cache file {:?} to final cache file {:?}",
-                incomplete_cache_path, cache_path
-            )
-        })?;
+    // Try atomic rename first, fall back to copy+delete if rename fails
+    match fs::rename(&incomplete_cache_path, cache_path).await {
+        Ok(()) => {
+            // Atomic rename succeeded
+        }
+        Err(rename_err) => {
+            // Rename failed, try copy + delete as fallback
+            if let Err(copy_err) = fs::copy(&incomplete_cache_path, cache_path).await {
+                return Err(anyhow::Error::new(copy_err).context(format!(
+                    "Failed to rename {incomplete_cache_path:?} to {cache_path:?} (rename error: {rename_err}) and copy fallback also failed"
+                )));
+            }
+
+            // Copy succeeded, now remove the incomplete file
+            if let Err(remove_err) = fs::remove_file(&incomplete_cache_path).await {
+                // Log warning but don't fail - the cache file was successfully created
+                warn!(
+                    "Failed to remove incomplete cache file {incomplete_cache_path:?} after copy: {remove_err}"
+                );
+            }
+        }
+    }
 
     // Delete the local file as its copy is now in the cache.
     info!("Deleting local file at {:?}", download_path);
     fs::remove_file(download_path)
         .await
-        .with_context(|| format!("Failed to delete local file {:?}", download_path))?;
+        .with_context(|| format!("Failed to delete local file {download_path:?}"))?;
 
     // Create a symlink from the cache file to the original download location.
     info!(
@@ -259,8 +378,7 @@ pub async fn handle_write_b10cache(download_path: &Path, cache_path: &Path) -> R
         .await
         .with_context(|| {
             format!(
-                "Failed to create symlink from cache file {:?} to local file path {:?}",
-                cache_path, download_path
+                "Failed to create symlink from cache file {cache_path:?} to local file path {download_path:?}"
             )
         })?;
 
@@ -276,7 +394,7 @@ pub async fn update_atime_by_reading(path: &Path) -> Result<()> {
     // Open the file in read-only mode.
     let mut file = fs::File::open(path)
         .await
-        .with_context(|| format!("Failed to open file {:?} for updating atime", path))?;
+        .with_context(|| format!("Failed to open file {path:?} for updating atime"))?;
     let mut buffer = [0u8; 1];
     let _ = file.read(&mut buffer).await?;
     Ok(())
