@@ -1,25 +1,24 @@
+use crate::create_basetenpointer;
+use anyhow::{anyhow, Context, Result};
+use log::{debug, error, info, warn};
 use std::collections::{HashMap, HashSet};
 use std::env;
 use std::path::{Path, PathBuf};
+use std::sync::atomic::{AtomicBool, Ordering};
 use std::sync::{Arc, Mutex, OnceLock};
-
-use anyhow::{anyhow, Context, Result};
-use chrono;
-use futures_util::stream::{FuturesUnordered, StreamExt};
-use log::{error, info, warn};
-use rand;
-use reqwest::Client;
-use serde_json;
-use serde_yaml;
 use tokio::fs;
-use tokio::sync::Semaphore;
+use tokio::sync::Mutex as TokioMutex;
+
+use tokio::sync::{mpsc, Semaphore};
+use tokio::task::JoinSet;
 
 use crate::bindings::{init_logger_once, resolve_truss_transfer_download_dir};
 use crate::cache::cleanup_b10cache_and_get_space_stats;
 use crate::constants::*;
 use crate::download::download_file_with_cache;
+use crate::metrics::{MetricEvent, MetricsGuard};
 use crate::speed_checks::is_b10cache_fast_heuristic;
-use crate::types::{BasetenPointer, BasetenPointerManifest, Resolution};
+use crate::types::{BasetenPointer, BasetenPointerManifest, ModelRepo, Resolution};
 
 // Global lock to serialize downloads (NOTE: this is process-local only)
 // For multi-process synchronization (e.g. in a "double start" scenario),
@@ -32,15 +31,19 @@ fn get_global_lock() -> &'static Arc<Mutex<()>> {
 }
 
 /// Shared entrypoint for both Python and CLI
-pub fn lazy_data_resolve_entrypoint(download_dir: Option<String>) -> Result<String> {
+pub fn lazy_data_resolve_entrypoint(
+    download_dir: Option<String>,
+    models: Option<Vec<ModelRepo>>,
+    model_path: String,
+) -> Result<String> {
     init_logger_once();
-    let num_workers = TRUSS_TRANSFER_NUM_WORKERS_DEFAULT;
+    let num_workers = *TRUSS_TRANSFER_NUM_WORKERS as usize;
     let download_dir = resolve_truss_transfer_download_dir(download_dir);
 
     // Ensure the global lock is initialized
     let lock = get_global_lock();
 
-    info!("Acquiring global download lock...");
+    info!("truss_transfer_cli, version: {}", env!("CARGO_PKG_VERSION"));
     let _guard = lock
         .lock()
         .map_err(|_| anyhow!("Global lock was poisoned"))?;
@@ -52,20 +55,65 @@ pub fn lazy_data_resolve_entrypoint(download_dir: Option<String>) -> Result<Stri
         .build()
         .context("Failed to build Tokio runtime")?;
 
-    // Run the asynchronous logic.
-    rt.block_on(async { lazy_data_resolve_async(download_dir.clone().into(), num_workers).await })?;
-    Ok(download_dir)
+    // Run the asynchronous logic with metrics.
+    let result = rt.block_on(async {
+        lazy_data_resolve_async(download_dir.clone().into(), num_workers, models, model_path).await
+    });
+
+    match result {
+        Ok(_) => Ok(download_dir),
+        Err(e) => Err(e),
+    }
 }
 
 /// Asynchronous implementation of the lazy data resolver logic.
-async fn lazy_data_resolve_async(download_dir: PathBuf, num_workers: usize) -> Result<()> {
-    info!("Checking for manifest files in multiple locations...");
+async fn lazy_data_resolve_async(
+    download_dir: PathBuf,
+    num_workers: usize,
+    models: Option<Vec<ModelRepo>>,
+    model_path: String,
+) -> Result<()> {
+    debug!("Checking for manifest files in multiple locations...");
 
-    let mut num_workers = num_workers;
+    // Initialize metrics collector
+    let metrics_guard = MetricsGuard::new();
+    let metrics_sender = metrics_guard
+        .sender()
+        .expect("Metrics sender should be available");
+
+    // Perform all operations - metrics_sender will be dropped at end of this function
+    let result = lazy_data_resolve_with_metrics(
+        download_dir,
+        num_workers,
+        models,
+        model_path,
+        metrics_sender,
+    )
+    .await;
+
+    // Finalize and flush metrics (after metrics_sender is dropped)
+    metrics_guard.finalize().await?;
+
+    result
+}
+
+/// Internal implementation that performs the actual download logic with metrics
+async fn lazy_data_resolve_with_metrics(
+    download_dir: PathBuf,
+    num_workers: usize,
+    models: Option<Vec<ModelRepo>>,
+    model_path: String,
+    metrics_sender: mpsc::UnboundedSender<MetricEvent>,
+) -> Result<()> {
+    let timer = tokio::time::Instant::now();
 
     // 1. Check multiple manifest locations and collect all available manifests
     let mut all_manifests = Vec::new();
     let mut found_paths = Vec::new();
+    let mut jit_models = Vec::new();
+    if let Some(m) = models {
+        jit_models.extend(m);
+    }
 
     for manifest_path_str in LAZY_DATA_RESOLVER_PATHS {
         let manifest_path = Path::new(manifest_path_str);
@@ -76,24 +124,32 @@ async fn lazy_data_resolve_async(download_dir: PathBuf, num_workers: usize) -> R
             // 2. Parse YAML/JSON asynchronously
             let file_data = fs::read_to_string(manifest_path)
                 .await
-                .with_context(|| format!("Unable to read manifest from {}", manifest_path_str))?;
+                .with_context(|| format!("Unable to read manifest from {manifest_path_str}"))?;
 
             // todo: try both JSON and YAML parsing
             // If it fails, we will try the other format
             let bptr_manifest: BasetenPointerManifest = if manifest_path_str.ends_with(".json") {
                 serde_json::from_str(&file_data).with_context(|| {
-                    format!("Failed to parse JSON manifest from {}", manifest_path_str)
+                    format!("Failed to parse JSON manifest from {manifest_path_str}")
                 })?
             } else {
                 serde_yaml::from_str(&file_data).with_context(|| {
-                    format!("Failed to parse YAML manifest from {}", manifest_path_str)
+                    format!("Failed to parse YAML manifest from {manifest_path_str}")
                 })?
             };
+            if bptr_manifest.models.is_some() {
+                jit_models.extend(bptr_manifest.models.clone().unwrap());
+            }
 
             all_manifests.push(bptr_manifest);
         }
     }
 
+    let manifest_jit = create_basetenpointer(jit_models, model_path)
+        .await
+        .map_err(|e| anyhow::anyhow!("{}", e))?;
+    all_manifests.push(manifest_jit);
+
     if all_manifests.is_empty() {
         return Err(anyhow!(
             "No manifest files found at any of the following locations: {}. Please ensure at least one manifest file exists before running.",
@@ -114,26 +170,23 @@ async fn lazy_data_resolve_async(download_dir: PathBuf, num_workers: usize) -> R
         merged_manifest.pointers.len()
     );
 
+    // Record manifest size
+    let total_manifest_size: u64 = merged_manifest.pointers.iter().map(|p| p.size).sum();
+    let _ = metrics_sender.send(MetricEvent::ManifestSize(total_manifest_size));
+
     // 4. Validate expiration and build the resolution map
     let mut resolution_map = build_resolution_map(&merged_manifest)?;
-    info!("All pointers validated successfully.");
+    debug!("All pointers validated successfully.");
 
     // 5. Check if b10cache is enabled
-    let allowed_b10_cache = match env::var(BASETEN_FS_ENABLED_ENV_VAR)
-        .unwrap_or_else(|_| "false".into())
-        .to_lowercase()
-        .as_str()
-    {
-        "1" | "true" => true,
-        _ => false,
-    };
+    let allowed_b10_cache = *BASETEN_FS_ENABLED;
     let mut read_from_b10cache = allowed_b10_cache;
     let mut write_to_b10cache = allowed_b10_cache;
 
     if allowed_b10_cache {
         info!("b10cache is enabled.");
         // create cache directory if it doesn't exist
-        fs::create_dir_all(CACHE_DIR)
+        fs::create_dir_all(&*CACHE_DIR)
             .await
             .context("Failed to create b10cache directory")?;
         // shuffle the resolution map to randomize the order of downloads
@@ -181,7 +234,7 @@ async fn lazy_data_resolve_async(download_dir: PathBuf, num_workers: usize) -> R
 
         // todo: check speed of b10cache (e.g. is faster than 100MB/s)
         // and stop using b10cache if download speed is faster
-        match is_b10cache_fast_heuristic(&merged_manifest).await {
+        match is_b10cache_fast_heuristic(&merged_manifest, metrics_sender.clone()).await {
             Ok(speed) => {
                 if speed {
                     info!("b10cache is faster than downloading.");
@@ -195,76 +248,217 @@ async fn lazy_data_resolve_async(download_dir: PathBuf, num_workers: usize) -> R
             }
         }
 
-        // only use at max 2 workers for b10cache, to avoid conflicts on parallel writes
-        if write_to_b10cache {
-            num_workers = num_workers.min(2);
-        }
         info!(
             "b10cache use: Read: {}, Write: {}",
             read_from_b10cache, write_to_b10cache
         );
+
+        // Record b10fs decision
+        let _ = metrics_sender.send(MetricEvent::B10fsDecision(read_from_b10cache));
     } else {
         info!("b10cache is not enabled for read or write.");
+        let _ = metrics_sender.send(MetricEvent::B10fsDecision(false));
     }
 
     // 6. Build concurrency limit
     info!("Using {} concurrent workers.", num_workers);
 
-    let semaphore = Arc::new(Semaphore::new(num_workers));
-    let client = Client::builder()
-        // https://github.com/hyperium/hyper/issues/2136#issuecomment-589488526
-        .tcp_keepalive(std::time::Duration::from_secs(15))
-        .http2_keep_alive_interval(Some(std::time::Duration::from_secs(15)))
-        .timeout(std::time::Duration::from_secs(BLOB_DOWNLOAD_TIMEOUT_SECS))
-        .build()?;
-
+    let semaphore_download = Arc::new(Semaphore::new(num_workers));
+    let semaphore_range_dw = Arc::new(Semaphore::new(*TRUSS_TRANSFER_RANGE_DOWNLOAD_WORKERS));
+    let lock_pre_page = Arc::new(TokioMutex::new(()));
     // resolve the gcs / s3 and pre-sign the urls
     // 6.1 TODO: create features for this to pre-sign url at runtime.
 
     // 7. Spawn download tasks
-    info!("Spawning download tasks...");
-    let mut tasks: FuturesUnordered<
-        tokio::task::JoinHandle<std::result::Result<(), anyhow::Error>>,
-    > = FuturesUnordered::new();
+    debug!("Spawning download tasks...");
+    let mut download_tasks = JoinSet::new();
+    let mut page_tasks: JoinSet<Result<(), anyhow::Error>> = JoinSet::new();
+
     for (file_name, pointer) in resolution_map {
         let download_dir = download_dir.clone();
-        let client = client.clone();
-        let sem_clone = semaphore.clone();
-        tasks.push(tokio::spawn(async move {
-            let _permit = sem_clone.acquire_owned().await;
+        let sem_clone = semaphore_download.clone();
+        let semaphore_range_dw_clone = semaphore_range_dw.clone();
+        let metrics_sender_clone = metrics_sender.clone();
+        download_tasks.spawn(async move {
+            let _permit = sem_clone.acquire_owned().await?;
             log::debug!("Handling file: {}", file_name);
             download_file_with_cache(
-                &client,
                 &pointer,
                 &download_dir,
                 &file_name,
                 read_from_b10cache,
                 write_to_b10cache,
+                num_workers,
+                semaphore_range_dw_clone,
+                metrics_sender_clone,
             )
             .await
-        }));
+        });
     }
 
-    // 8. Await all tasks
-    info!("Awaiting completion of all download tasks...");
-    while let Some(join_result) = tasks.next().await {
+    // 8. Await all tasks, and cancel remaining on first failure.
+    debug!("Awaiting completion of all download tasks...");
+    while let Some(join_result) = download_tasks.join_next().await {
         match join_result {
-            Ok(Ok(())) => {} // task succeeded
+            Ok(Ok(file_name)) => {
+                // Task succeeded. If PAGE after download is active, load the file into memory.
+                if *TRUSS_TRANSFER_PAGE_AFTER_DOWNLOAD {
+                    let lock_clone = lock_pre_page.clone();
+                    let file_path = download_dir.join(&file_name);
+
+                    page_tasks.spawn(async move {
+                        tokio::time::sleep(std::time::Duration::from_millis(500)).await;
+                        page_file_into_memory(&file_path, lock_clone).await;
+                        Ok(())
+                    });
+                }
+            }
             Ok(Err(e)) => {
+                // A download task returned an error. Abort all other tasks.
                 error!("A download task failed: {}", e);
+                download_tasks.abort_all();
+                page_tasks.abort_all();
+                let _ = metrics_sender.send(MetricEvent::Success(false));
                 return Err(anyhow!("Download failure: {}", e));
             }
             Err(e) => {
-                error!("A Tokio task panicked: {}", e);
-                return Err(anyhow!("Tokio task panicked: {}", e));
+                // A Tokio task paniced. Abort all other tasks.
+                error!("A Tokio task paniced: {}", e);
+                download_tasks.abort_all();
+                page_tasks.abort_all();
+                let _ = metrics_sender.send(MetricEvent::Success(false));
+                return Err(anyhow!("Tokio task paniced: {}", e));
             }
         }
     }
 
-    info!("All downloads completed successfully!");
+    info!(
+        "All downloads completed successfully after {:?}",
+        timer.elapsed()
+    );
+
+    // Mark as successful
+    let _ = metrics_sender.send(MetricEvent::Success(true));
+
+    if !page_tasks.is_empty() {
+        debug!(
+            "Waiting for up to 1 seconds for {} remaining paging tasks to complete...",
+            page_tasks.len()
+        );
+        let deadline = tokio::time::Instant::now() + std::time::Duration::from_secs(1);
+        while let Ok(Some(join_result)) =
+            tokio::time::timeout_at(deadline, page_tasks.join_next()).await
+        {
+            if let Err(e) = join_result {
+                warn!("A paging task paniced: {}", e);
+            }
+        }
+    }
+
+    // If any paging tasks are still running after the timeout, abort them.
+    if !page_tasks.is_empty() {
+        debug!(
+            "Timeout reached. Aborting {} remaining paging tasks.",
+            page_tasks.len()
+        );
+        page_tasks.abort_all();
+    }
+
     Ok(())
 }
 
+/// A helper struct that calls a function when it's dropped.
+struct DropGuard<F: FnOnce()> {
+    func: Option<F>,
+}
+
+impl<F: FnOnce()> DropGuard<F> {
+    fn new(func: F) -> Self {
+        Self { func: Some(func) }
+    }
+}
+
+impl<F: FnOnce()> Drop for DropGuard<F> {
+    fn drop(&mut self) {
+        if let Some(func) = self.func.take() {
+            func();
+        }
+    }
+}
+
+/// Reads a file to load it into the OS page cache.
+/// This version uses a large buffer and runs the I/O operations in a blocking thread
+/// to avoid stalling the async runtime.
+async fn page_file_into_memory(path: &Path, lock: Arc<TokioMutex<()>>) {
+    let cancel_flag = Arc::new(AtomicBool::new(false));
+
+    // This guard ensures the flag is set if this async function is dropped (e.g., by task abortion).
+    let _guard = DropGuard::new({
+        let flag_clone = cancel_flag.clone();
+        move || {
+            flag_clone.store(true, Ordering::Relaxed);
+        }
+    });
+
+    // Acquire the lock to ensure only one paging operation happens at a time.
+    let _guard = lock.lock().await;
+    debug!("Paging {} into memory", path.display());
+    let path_owned = path.to_path_buf();
+
+    let result = tokio::task::spawn_blocking(move || -> Result<(), anyhow::Error> {
+        use std::fs::File;
+        use std::io::Read;
+
+        let mut file = File::open(&path_owned)
+            .with_context(|| format!("Failed to open file for paging: {}", path_owned.display()))?;
+
+        // Use a large buffer for more efficient disk reads
+        // 128 MB is a good starting point.
+        const BUFFER_SIZE: u64 = 128 * 1024 * 1024;
+        // check buffer size or file_size metadata
+        let file_len = file.metadata().map(|m| m.len()).unwrap_or(0);
+        let buffer_size = file_len.min(BUFFER_SIZE) as usize;
+
+        let mut buffer = vec![0; buffer_size];
+
+        // Read the file chunk by chunk to load it into the OS page cache.
+        while file.read(&mut buffer)? > 0 {
+            if cancel_flag.load(Ordering::Relaxed) {
+                info!("Paging for file {} cancelled.", path_owned.display());
+                return Err(anyhow!("Paging operation was cancelled."));
+            }
+            // throttle: Lower contention on disk and give priority to more important tasks.
+            std::thread::sleep(std::time::Duration::from_millis(25));
+        }
+        if file_len < BUFFER_SIZE {
+            debug!("Finished paging file {} into memory", path_owned.display());
+        } else {
+            info!("Finished paging file {} into memory", path_owned.display());
+        }
+
+        Ok(())
+    })
+    .await;
+
+    match result {
+        Ok(Ok(_)) => { /* Paging completed successfully */ }
+        Ok(Err(e)) => {
+            // Log errors that are not cancellation-related.
+            if !e.to_string().contains("Paging operation was cancelled.") {
+                error!("Failed to page file {} into memory: {}", path.display(), e);
+            }
+        }
+        Err(e) => {
+            // The spawn_blocking task itself paniced or was cancelled.
+            if e.is_cancelled() {
+                warn!("Paging task for {} was cancelled.", path.display());
+            } else {
+                error!("Paging task for {} paniced: {}", path.display(), e);
+            }
+        }
+    }
+}
+
 /// Validate expiration and build a vector of (file_name, (URL, hash, size)).
 pub fn build_resolution_map(
     bptr_manifest: &BasetenPointerManifest,
@@ -276,7 +470,10 @@ pub fn build_resolution_map(
         match &bptr.resolution {
             Resolution::Http(http_resolution) => {
                 if http_resolution.expiration_timestamp < now {
-                    return Err(anyhow!("Baseten pointer lazy data resolution has expired"));
+                    error!(
+                        "Pointer {} has expired at {}. Current time is {}. This will lead to a download failure.",
+                        bptr.file_name, http_resolution.expiration_timestamp, now
+                    );
                 }
             }
             _ => {
@@ -333,7 +530,7 @@ pub fn merge_manifests(manifests: Vec<BasetenPointerManifest>) -> Result<Baseten
         }
     }
 
-    info!(
+    debug!(
         "Merged {} pointers from {} manifests",
         merged_pointers.len(),
         manifests_count
@@ -341,5 +538,6 @@ pub fn merge_manifests(manifests: Vec<BasetenPointerManifest>) -> Result<Baseten
 
     Ok(BasetenPointerManifest {
         pointers: merged_pointers,
+        models: None,
     })
 }
