@@ -1,13 +1,14 @@
 use anyhow::{anyhow, Result};
 
+use crate::create::object_storage_client::get_client_options;
 use crate::secrets::get_secret_from_file;
 
 /// Parse Azure Blob Storage URI into account, container, and blob components
 /// Expected format: azure://accountname/containername/path/to/blob
 /// or https://accountname.blob.core.windows.net/containername/path/to/blob
 pub fn parse_azure_uri(uri: &str) -> Result<(String, String, String)> {
-    if uri.starts_with("azure://") {
-        let without_prefix = &uri[8..]; // Remove "azure://"
+    if let Some(without_prefix) = uri.strip_prefix("azure://") {
+        // Remove "azure://"
         let parts: Vec<&str> = without_prefix.splitn(3, '/').collect();
 
         if parts.len() != 3 {
@@ -69,7 +70,7 @@ pub fn parse_azure_uri(uri: &str) -> Result<(String, String, String)> {
 /// Azure credentials structure for parsing from single file
 #[derive(Debug, serde::Deserialize)]
 struct AzureCredentials {
-    account_key: Option<String>,
+    account_key: String,
     #[serde(default)]
     use_emulator: bool,
 }
@@ -82,15 +83,16 @@ pub fn azure_storage(
 ) -> Result<Box<dyn object_store::ObjectStore>, anyhow::Error> {
     use object_store::azure::{MicrosoftAzure, MicrosoftAzureBuilder};
 
-    let mut builder = MicrosoftAzureBuilder::new().with_account(account_name);
+    let mut builder = MicrosoftAzureBuilder::new()
+        .with_account(account_name)
+        .with_client_options(get_client_options());
 
     // Read Azure credentials from single file
     if let Some(credentials_content) = get_secret_from_file(runtime_secret_name) {
         // Try to parse as JSON first
         if let Ok(credentials) = serde_json::from_str::<AzureCredentials>(&credentials_content) {
-            if let Some(account_key) = credentials.account_key {
-                builder = builder.with_access_key(account_key);
-            }
+            builder = builder.with_access_key(credentials.account_key);
+
             // Note: SAS token support would require proper URL parsing and query parameter handling
             // For now, we focus on access key authentication
             // else if let Some(sas_token) = credentials.sas_token {
@@ -101,33 +103,13 @@ pub fn azure_storage(
                 builder = builder.with_use_emulator(true);
             }
         } else {
-            // Fallback: try to parse as simple key=value format
-            for line in credentials_content.lines() {
-                let line = line.trim();
-                if line.is_empty() || line.starts_with('#') {
-                    continue;
-                }
-
-                if let Some((key, value)) = line.split_once('=') {
-                    match key.trim().to_lowercase().as_str() {
-                        "account_key" | "azure_storage_account_key" => {
-                            builder = builder.with_access_key(value.trim());
-                        }
-                        "sas_token" | "azure_storage_sas_token" => {
-                            // Note: SAS token support would require proper URL parsing
-                            // For now, skip SAS token in key=value format
-                            // builder = builder.with_sas_token(value.trim());
-                        }
-                        "use_emulator" => {
-                            if value.trim().to_lowercase() == "true" {
-                                builder = builder.with_use_emulator(true);
-                            }
-                        }
-                        _ => {} // Ignore unknown keys
-                    }
-                }
-            }
+            return Err(anyhow!("Failed to parse Azure credentials from JSON. The json needs to be in the format: {{\"account_key\": \"...\"}}"));
         }
+    } else {
+        return Err(anyhow!(
+            "Failed to read Azure credentials from not existing file: {}",
+            runtime_secret_name
+        ));
     }
 
     let azure: MicrosoftAzure = builder
