@@ -1,13 +1,14 @@
+use crate::create::object_storage_client::get_client_options;
 use crate::types::GcsError;
 use object_store::gcp::GoogleCloudStorageBuilder;
-use std::fs;
+
+use crate::secrets::get_secret_path;
 
 /// Parse GCS URI (gs://bucket/path) into bucket and prefix
 pub fn parse_gcs_uri(uri: &str) -> Result<(String, String), GcsError> {
     if !uri.starts_with("gs://") {
         return Err(GcsError::InvalidUri(format!(
-            "URI must start with gs://: {}",
-            uri
+            "URI must start with gs://: {uri}"
         )));
     }
 
@@ -28,12 +29,12 @@ pub fn gcs_storage(
     bucket: &str,
     runtime_secret_name: &str,
 ) -> Result<object_store::gcp::GoogleCloudStorage, GcsError> {
-    let secret_path = format!("/secrets/{}", runtime_secret_name);
-    let credentials_json = fs::read_to_string(&secret_path).map_err(|e| GcsError::Io(e))?;
+    let path = get_secret_path(runtime_secret_name);
 
     GoogleCloudStorageBuilder::new()
-        .with_service_account_key(&credentials_json)
+        .with_service_account_path(&path)
         .with_bucket_name(bucket)
+        .with_client_options(get_client_options())
         .build()
         .map_err(GcsError::ObjectStore)
 }
