@@ -51,8 +51,12 @@ impl CloudMetadataProvider for AwsProvider {
             .unwrap_or_else(|| format!("s3-{}", rand::random::<u64>()))
     }
 
+    fn last_modified_time(&self, meta: &object_store::ObjectMeta) -> chrono::DateTime<chrono::Utc> {
+        meta.last_modified
+    }
+
     fn generate_uid(&self, bucket: &str, object_path: &str, _hash: &str) -> String {
-        format!("s3:{}:{}", bucket, object_path)
+        format!("s3:{bucket}:{object_path}")
     }
 }
 
@@ -66,11 +70,15 @@ impl StorageProvider for AwsProvider {
         repo.repo_id.starts_with("s3://") || matches!(repo.kind, ResolutionType::S3)
     }
 
-    async fn create_pointers(&self, repo: &ModelRepo) -> Result<Vec<BasetenPointer>> {
+    async fn create_pointers(
+        &self,
+        repo: &ModelRepo,
+        model_path: &String,
+    ) -> Result<Vec<BasetenPointer>> {
         debug!("Creating AWS S3 pointers for repo: {}", repo.repo_id);
         if !self.can_handle(repo) {
             warn!("AWS S3 provider cannot handle repo: {}", repo.repo_id);
         }
-        create_single_cloud_basetenpointers(self, repo).await
+        create_single_cloud_basetenpointers(self, repo, model_path).await
     }
 }
