@@ -23,7 +23,7 @@ impl CloudMetadataProvider for AzureProvider {
     fn parse_uri(&self, uri: &str) -> Result<(String, String)> {
         let (account, container, blob_prefix) = parse_azure_uri(uri)?;
         // Combine account and container as "bucket" for consistency
-        let bucket = format!("{}/{}", account, container);
+        let bucket = format!("{account}/{container}");
         Ok((bucket, blob_prefix))
     }
 
@@ -41,7 +41,7 @@ impl CloudMetadataProvider for AzureProvider {
     fn create_resolution(&self, bucket: &str, object_path: &str) -> Resolution {
         // Extract account and container from bucket
         let parts: Vec<&str> = bucket.split('/').collect();
-        let account = parts.get(0).unwrap_or(&"").to_string();
+        let account = parts.first().unwrap_or(&"").to_string();
         let container = parts.get(1).unwrap_or(&"").to_string();
 
         Resolution::Azure(AzureResolution::new(
@@ -60,14 +60,17 @@ impl CloudMetadataProvider for AzureProvider {
             .clone()
             .unwrap_or_else(|| format!("azure-{}", rand::random::<u64>()))
     }
+    fn last_modified_time(&self, meta: &object_store::ObjectMeta) -> chrono::DateTime<chrono::Utc> {
+        meta.last_modified
+    }
 
     fn generate_uid(&self, bucket: &str, object_path: &str, _hash: &str) -> String {
         // Extract account and container for UID
         let parts: Vec<&str> = bucket.split('/').collect();
-        let account = parts.get(0).unwrap_or(&"");
+        let account = parts.first().unwrap_or(&"");
         let container = parts.get(1).unwrap_or(&"");
 
-        format!("azure:{}:{}:{}", account, container, object_path)
+        format!("azure:{account}:{container}:{object_path}")
     }
 }
 
@@ -83,7 +86,11 @@ impl StorageProvider for AzureProvider {
             || matches!(repo.kind, ResolutionType::Azure)
     }
 
-    async fn create_pointers(&self, repo: &ModelRepo) -> Result<Vec<BasetenPointer>> {
+    async fn create_pointers(
+        &self,
+        repo: &ModelRepo,
+        model_path: &String,
+    ) -> Result<Vec<BasetenPointer>> {
         debug!(
             "Creating Azure Blob Storage pointers for repo: {}",
             repo.repo_id
@@ -94,6 +101,6 @@ impl StorageProvider for AzureProvider {
                 repo.repo_id
             );
         }
-        create_single_cloud_basetenpointers(self, repo).await
+        create_single_cloud_basetenpointers(self, repo, model_path).await
     }
 }
