@@ -59,8 +59,12 @@ impl CloudMetadataProvider for GcsProvider {
             .unwrap_or_else(|| format!("gcs-{}", rand::random::<u64>()))
     }
 
+    fn last_modified_time(&self, meta: &object_store::ObjectMeta) -> chrono::DateTime<chrono::Utc> {
+        meta.last_modified
+    }
+
     fn generate_uid(&self, _bucket: &str, _object_path: &str, hash: &str) -> String {
-        format!("gcs-{}", hash)
+        format!("gcs-{hash}")
     }
 }
 
@@ -74,11 +78,15 @@ impl StorageProvider for GcsProvider {
         repo.repo_id.starts_with("gs://") || matches!(repo.kind, ResolutionType::Gcs)
     }
 
-    async fn create_pointers(&self, repo: &ModelRepo) -> Result<Vec<BasetenPointer>> {
+    async fn create_pointers(
+        &self,
+        repo: &ModelRepo,
+        model_path: &String,
+    ) -> Result<Vec<BasetenPointer>> {
         debug!("Creating GCS pointers for repo: {}", repo.repo_id);
         if !self.can_handle(repo) {
             warn!("GCS provider cannot handle repo: {}", repo.repo_id);
         }
-        create_single_cloud_basetenpointers(self, repo).await
+        create_single_cloud_basetenpointers(self, repo, model_path).await
     }
 }
