@@ -1,4 +1,6 @@
 import json
+import pathlib
+import shutil
 
 import truss_transfer
 
@@ -137,7 +139,7 @@ def test_dolly():
     print("Success! Generated BasetenPointer manifest:")
 
     # Parse and pretty print the JSON
-    manifest = json.loads(result)
+    manifest = json.loads(result)["pointers"]
     print(json.dumps(manifest, indent=2))
 
     # Test that the structure is correct
@@ -200,3 +202,26 @@ def test_qwen3():
     manifest = json.loads(result)
 
     print(json.dumps(manifest, indent=2))
+
+
+def test_direct_download():
+    models = [
+        truss_transfer.PyModelRepo(
+            repo_id="julien-c/dummy-unknown",
+            revision="main",
+            runtime_secret_name="none",
+            volume_folder="julien_dummy",
+            kind="hf",
+        )
+    ]
+    shutil.rmtree("/tmp/data", ignore_errors=True)
+
+    truss_transfer.lazy_data_resolve("", models, "/tmp/data")
+    assert pathlib.Path("/tmp/data/julien_dummy/config.json").exists()
+    assert pathlib.Path("/tmp/data/julien_dummy/config.json").stat().st_size == 496
+
+    # print(json.dumps(manifest, indent=2))
+
+
+if __name__ == "__main__":
+    test_direct_download()
